# OPTLAB - Copyright (C) 2018 E. Kuci, C. Geuzaine and P. Duysinx, University of Liege
#
# See the LICENSE.txt file for license information. Please report all
# bugs and problems to the public mailing list <optlab@onelab.info>.

# This file defines the OPTLAB Julia API (v1.0).
#
# Do not edit it directly: it is automatically generated by `api/gen.py'.
#
# By design, the OPTLAB Julia API is purely functional, and only uses elementary
# types. See `demos/api' for examples.

"""
    module optlab

Top-level functions
"""
module optlab

const OPTLAB_API_VERSION = "1.0"
const libdir = dirname(@__FILE__)
const lib = joinpath(libdir, is_windows() ? "optlab-1.0" : "liboptlab")

"""
    optlab.initialize(type = "none")

Initialize SCP.
"""
function initialize(type = "none")
    ierr = Ref{Cint}()
    ccall((:optlabInitialize, lib), Void,
          (Ptr{Cchar}, Ptr{Cint}),
          type, ierr)
    ierr[] != 0 && error("optlabInitialize returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.run()

Solve the optimization problem.
"""
function run()
    ierr = Ref{Cint}()
    ccall((:optlabRun, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabRun returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.addObjective(func)

Add the objective function.
"""
function addObjective(func)
    ierr = Ref{Cint}()
    ccall((:optlabAddObjective, lib), Void,
          (Ptr{Void}, Ptr{Cint}),
          func, ierr)
    ierr[] != 0 && error("optlabAddObjective returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.addConstraint()

Solve the optimization problem.
"""
function addConstraint()
    ierr = Ref{Cint}()
    ccall((:optlabAddConstraint, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabAddConstraint returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.addGroupOfConstraints()

Solve the optimization problem.
"""
function addGroupOfConstraints()
    ierr = Ref{Cint}()
    ccall((:optlabAddGroupOfConstraints, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabAddGroupOfConstraints returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.addVariable()

Solve the optimization problem.
"""
function addVariable()
    ierr = Ref{Cint}()
    ccall((:optlabAddVariable, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabAddVariable returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.addGroupOfVariables()

Solve the optimization problem.
"""
function addGroupOfVariables()
    ierr = Ref{Cint}()
    ccall((:optlabAddGroupOfVariables, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabAddGroupOfVariables returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.computeObjective()

Compute the objective function.
"""
function computeObjective()
    ierr = Ref{Cint}()
    ccall((:optlabComputeObjective, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabComputeObjective returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.finalize()

Finalize Optlab.
"""
function finalize()
    ierr = Ref{Cint}()
    ccall((:optlabFinalize, lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabFinalize returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    module optlab.option

Global option handling functions
"""
module option

import ..optlab

"""
    optlab.option.setNumber(name, value)

Set a numerical option to `value`. `name`. Available categories and options are
listed in the Scp reference manual.
"""
function setNumber(name, value)
    ierr = Ref{Cint}()
    ccall((:optlabOptionSetNumber, optlab.lib), Void,
          (Ptr{Cchar}, Cdouble, Ptr{Cint}),
          name, value, ierr)
    ierr[] != 0 && error("optlabOptionSetNumber returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.option.getNumber(name)

Get the `value` of a numerical option.

Return `value`.
"""
function getNumber(name)
    api_value_ = Ref{Cdouble}()
    ierr = Ref{Cint}()
    ccall((:optlabOptionGetNumber, optlab.lib), Void,
          (Ptr{Cchar}, Ptr{Cdouble}, Ptr{Cint}),
          name, api_value_, ierr)
    ierr[] != 0 && error("optlabOptionGetNumber returned non-zero error code: $(ierr[])")
    return api_value_[]
end

"""
    optlab.option.show()

Print the numerical options.
"""
function show()
    ierr = Ref{Cint}()
    ccall((:optlabOptionShow, optlab.lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabOptionShow returned non-zero error code: $(ierr[])")
    return nothing
end

end # end of module option

"""
    module optlab.mma

MMA optimizer
"""
module mma

import ..optlab

"""
    optlab.mma.initialize(currentPoint, lowerBound, upperBound, restart = false)

Initialize MMA. `numberOfConstraints` ... FIXME TODO
"""
function initialize(currentPoint, lowerBound, upperBound, restart = false)
    ierr = Ref{Cint}()
    ccall((:optlabMmaInitialize, optlab.lib), Void,
          (Ptr{Cdouble}, Csize_t, Ptr{Cdouble}, Csize_t, Ptr{Cdouble}, Csize_t, Cint, Ptr{Cint}),
          currentPoint, length(currentPoint), lowerBound, length(lowerBound), upperBound, length(upperBound), restart, ierr)
    ierr[] != 0 && error("optlabMmaInitialize returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.mma.getCurrentPoint()

Get current point.

Return `currentPoint`.
"""
function getCurrentPoint()
    api_currentPoint_ = Ref{Ptr{Cdouble}}()
    api_currentPoint_n_ = Ref{Csize_t}()
    ierr = Ref{Cint}()
    ccall((:optlabMmaGetCurrentPoint, optlab.lib), Void,
          (Ptr{Ptr{Cdouble}}, Ptr{Csize_t}, Ptr{Cint}),
          api_currentPoint_, api_currentPoint_n_, ierr)
    ierr[] != 0 && error("optlabMmaGetCurrentPoint returned non-zero error code: $(ierr[])")
    currentPoint = unsafe_wrap(Array, api_currentPoint_[], api_currentPoint_n_[], true)
    return currentPoint
end

"""
    optlab.mma.updateCurrentPoint(constraints, gradientOfObjective, gradientOfConstraints, diagHessianOfObjective = Cdouble[])

Update current point (based on first order derivatives and possibly the diagonal
entries of the objective function hessian).
"""
function updateCurrentPoint(constraints, gradientOfObjective, gradientOfConstraints, diagHessianOfObjective = Cdouble[])
    ierr = Ref{Cint}()
    ccall((:optlabMmaUpdateCurrentPoint, optlab.lib), Void,
          (Ptr{Cdouble}, Csize_t, Ptr{Cdouble}, Csize_t, Ptr{Cdouble}, Csize_t, Ptr{Cdouble}, Csize_t, Ptr{Cint}),
          constraints, length(constraints), gradientOfObjective, length(gradientOfObjective), gradientOfConstraints, length(gradientOfConstraints), diagHessianOfObjective, length(diagHessianOfObjective), ierr)
    ierr[] != 0 && error("optlabMmaUpdateCurrentPoint returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.mma.getOuterIteration()

Get outer iteration.

Return `iteration`.
"""
function getOuterIteration()
    api_iteration_ = Ref{Cint}()
    ierr = Ref{Cint}()
    ccall((:optlabMmaGetOuterIteration, optlab.lib), Void,
          (Ptr{Cint}, Ptr{Cint}),
          api_iteration_, ierr)
    ierr[] != 0 && error("optlabMmaGetOuterIteration returned non-zero error code: $(ierr[])")
    return api_iteration_[]
end

"""
    optlab.mma.getDesignChange()

Get the infinite norm between two successive designs.

Return `change`.
"""
function getDesignChange()
    api_change_ = Ref{Cdouble}()
    ierr = Ref{Cint}()
    ccall((:optlabMmaGetDesignChange, optlab.lib), Void,
          (Ptr{Cdouble}, Ptr{Cint}),
          api_change_, ierr)
    ierr[] != 0 && error("optlabMmaGetDesignChange returned non-zero error code: $(ierr[])")
    return api_change_[]
end

"""
    optlab.mma.setMoveLimits(lowerBound, upperBound, move)

Set the outer move limits.
"""
function setMoveLimits(lowerBound, upperBound, move)
    ierr = Ref{Cint}()
    ccall((:optlabMmaSetMoveLimits, optlab.lib), Void,
          (Ptr{Cdouble}, Csize_t, Ptr{Cdouble}, Csize_t, Cdouble, Ptr{Cint}),
          lowerBound, length(lowerBound), upperBound, length(upperBound), move, ierr)
    ierr[] != 0 && error("optlabMmaSetMoveLimits returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.mma.finalize()

Finalize MMA.
"""
function finalize()
    ierr = Ref{Cint}()
    ccall((:optlabMmaFinalize, optlab.lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabMmaFinalize returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    module optlab.mma.option

Global option handling functions
"""
module option

import ...optlab

"""
    optlab.mma.option.setNumber(name, value)

Set a numerical option to `value`. `name`. Available categories and options are
listed in the Scp reference manual.
"""
function setNumber(name, value)
    ierr = Ref{Cint}()
    ccall((:optlabMmaOptionSetNumber, optlab.lib), Void,
          (Ptr{Cchar}, Cdouble, Ptr{Cint}),
          name, value, ierr)
    ierr[] != 0 && error("optlabMmaOptionSetNumber returned non-zero error code: $(ierr[])")
    return nothing
end

"""
    optlab.mma.option.getNumber(name)

Get the `value` of a numerical option.

Return `value`.
"""
function getNumber(name)
    api_value_ = Ref{Cdouble}()
    ierr = Ref{Cint}()
    ccall((:optlabMmaOptionGetNumber, optlab.lib), Void,
          (Ptr{Cchar}, Ptr{Cdouble}, Ptr{Cint}),
          name, api_value_, ierr)
    ierr[] != 0 && error("optlabMmaOptionGetNumber returned non-zero error code: $(ierr[])")
    return api_value_[]
end

"""
    optlab.mma.option.show()

Print the numerical options.
"""
function show()
    ierr = Ref{Cint}()
    ccall((:optlabMmaOptionShow, optlab.lib), Void,
          (Ptr{Cint},),
          ierr)
    ierr[] != 0 && error("optlabMmaOptionShow returned non-zero error code: $(ierr[])")
    return nothing
end

end # end of module option

end # end of module mma

end # end of module optlab
